@@ -1013,7 +1013,7 @@ void calc_closed(int i1, int j1, int i2, int j2)
      v7b = 65000;
      if ((x[k3]!=X)) { /*  && !((k4 > helix_start) && (i2 < helix_end-1))) { */
        /* normal internal loops: */
-       for (k3=i1+2; k3<=min(i1+3, j1-1); k3++) {
+       for (k3=i1+2; k3<=min(i1+min(3,iloop_upper_limit+1), j1-1); k3++) {
 	 if (x[k3]==X)
 	   break;
 	 for (k4=i2+4; k4<=min(i2+iloop_upper_limit+1, j2-1); k4++) {
@@ -1034,7 +1034,7 @@ void calc_closed(int i1, int j1, int i2, int j2)
 	 for (k3=i1+4; k3<=min(i1+iloop_upper_limit+1, j1-1); k3++) {
 	   if (x[k3]==X)
 	     break;
-	   for (k4=i2+2; k4<=min(i2+3, j2-1); k4++) {
+	   for (k4=i2+2; k4<=min(i2+min(3,iloop_upper_limit+1), j2-1); k4++) {
 	     if ((k4 > helix_start) && (i2 < helix_end-1))
 	       break;
 	     if (compl(x[k3+1],y[k4+1])) {
@@ -1448,7 +1448,7 @@ struct str1 back_closed(int i1, int j1, int i2, int j2)
      if ((x[k3]!=X)) { /*  && !((k4 > helix_start) && (i2 < helix_end-1))) { */
 
        /* normal internal loops: */
-       for (k3=i1+2; k3<=min(i1+3, j1-1); k3++) {
+       for (k3=i1+2; k3<=min(i1+min(3,iloop_upper_limit+1), j1-1); k3++) {
 	 if (x[k3]==X)
 	   break;
 	 for (k4=i2+4; k4<=min(i2+iloop_upper_limit+1, j2-1); k4++) {
@@ -1475,7 +1475,7 @@ struct str1 back_closed(int i1, int j1, int i2, int j2)
 	 for (k3=i1+4; k3<=min(i1+iloop_upper_limit+1, j1-1); k3++) {
 	   if (x[k3]==X)
 	     break;
-	   for (k4=i2+2; k4<=min(i2+3, j2-1); k4++) {
+	   for (k4=i2+2; k4<=min(i2+min(3,iloop_upper_limit+1), j2-1); k4++) {
 	     if ((k4 > helix_start) && (i2 < helix_end-1))
 	       break;
 	     if (compl(x[k3+1],y[k4+1])) {
@@ -1680,6 +1680,10 @@ void mainloop(int bflag, int hit_number, int eflag, float energy_cutoff, int pfl
      hit_length,
      t_len;
 
+   int target_has_left_dangle;
+   int target_has_right_dangle;
+
+
    float
      normalised_energy,
      pvalue;
@@ -1707,7 +1711,6 @@ void mainloop(int bflag, int hit_number, int eflag, float energy_cutoff, int pfl
      /* ----------------------------- show axiom: hybrid ---------------------------- */
      v2 = calc_hybrid(0, m, 0, n);
 
-
      l = back_hybrid(0, m, 0, n);
 
 /*      gx = 1; */
@@ -1842,14 +1845,12 @@ void mainloop(int bflag, int hit_number, int eflag, float energy_cutoff, int pfl
        k = 0;
        while (k < t_len && t1[k]==' ' && t2[k]==' ')
 	 k++;
-       if (k<t_len)
-	 t1[k]=' '; /* hide left dangle */
+       target_has_left_dangle = k < t_len && t1[k] != ' ';
 
        k = t_len-1;
        while (k >= 0 && t1[k]==' ' && t2[k]==' ')
 	 k--;
-       if (k>=0)
-	 t1[k]=' '; /* hide right dangle */
+       target_has_right_dangle = k >=0 && t1[k] != ' ';
 
        /* count hit length: */
        hit_length = 0;
@@ -1862,11 +1863,15 @@ void mainloop(int bflag, int hit_number, int eflag, float energy_cutoff, int pfl
 	 if (t2[k]!=' ')
 	   hit_length++;
 
+      hit_length -= target_has_left_dangle + target_has_right_dangle;
+
 /*        printf("\n\nhitlength:%d\n\n",hit_length);  */
 
        /* mask hit: */
        for (k=0; k<hit_length; k++)
-	 x[gx+(t1[0]!=' ')+k]=X;
+/* 	 x[gx+(t1[0]!=' ')+k]=X; */
+	 x[gx+target_has_left_dangle+k]=X;
+
 
 /*        for (k=1; k<=m; k++) */
 /* 	 printf("%d",x[k]); */
